@@ -1,63 +1,53 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qthread.h"
 #include "qthreadstorage.h"
 #include "qmutex.h"
 #include "qreadwritelock.h"
 #include "qabstracteventdispatcher.h"
+#include "qbindingstorage.h"
 
 #include <qeventloop.h>
 
 #include "qthread_p.h"
 #include "private/qcoreapplication_p.h"
 
+#include <limits>
+
 QT_BEGIN_NAMESPACE
 
+/*
+    QPostEventList
+*/
+
+void QPostEventList::addEvent(const QPostEvent &ev)
+{
+    int priority = ev.priority;
+    if (isEmpty() ||
+            constLast().priority >= priority ||
+            insertionOffset >= size()) {
+        // optimization: we can simply append if the last event in
+        // the queue has higher or equal priority
+        append(ev);
+    } else {
+        // insert event in descending priority order, using upper
+        // bound for a given priority (to ensure proper ordering
+        // of events with the same priority)
+        QPostEventList::iterator at = std::upper_bound(begin() + insertionOffset, end(), ev);
+        insert(at, ev);
+    }
+}
+
+
 /*
   QThreadData
 */
 
 QThreadData::QThreadData(int initialRefCount)
     : _ref(initialRefCount), loopLevel(0), scopeLevel(0),
-      eventDispatcher(0),
+      eventDispatcher(nullptr),
       quitNow(false), canWait(true), isAdopted(false), requiresCoreApplication(true)
 {
     // fprintf(stderr, "QThreadData %p created\n", this);
@@ -65,7 +55,9 @@ QThreadData::QThreadData(int initialRefCount)
 
 QThreadData::~QThreadData()
 {
-    Q_ASSERT(_ref.load() == 0);
+#if QT_CONFIG(thread)
+    Q_ASSERT(_ref.loadRelaxed() == 0);
+#endif
 
     // In the odd case that Qt is running on a secondary thread, the main
     // thread instance will have been dereffed asunder because of the deref in
@@ -73,8 +65,8 @@ QThreadData::~QThreadData()
     // crashing during QCoreApplicationData's global static cleanup we need to
     // safeguard the main thread here.. This fix is a bit crude, but it solves
     // the problem...
-    if (this->thread == QCoreApplicationPrivate::theMainThread) {
-       QCoreApplicationPrivate::theMainThread = 0;
+    if (this->thread.loadAcquire() == QCoreApplicationPrivate::theMainThread.loadAcquire()) {
+       QCoreApplicationPrivate::theMainThread.storeRelease(nullptr);
        QThreadData::clearCurrentThreadData();
     }
 
@@ -85,15 +77,15 @@ QThreadData::~QThreadData()
     // because this destructor is still running (the _ref sub-object has not
     // been destroyed) and there's no reentrancy. The refcount will become
     // negative, but that's acceptable.
-    QThread *t = thread;
-    thread = 0;
+    QThread *t = thread.loadAcquire();
+    thread.storeRelease(nullptr);
     delete t;
 
     for (int i = 0; i < postEventList.size(); ++i) {
         const QPostEvent &pe = postEventList.at(i);
         if (pe.event) {
             --pe.receiver->d_func()->postedEvents;
-            pe.event->posted = false;
+            pe.event->m_posted = false;
             delete pe.event;
         }
     }
@@ -105,7 +97,7 @@ void QThreadData::ref()
 {
 #if QT_CONFIG(thread)
     (void) _ref.ref();
-    Q_ASSERT(_ref.load() != 0);
+    Q_ASSERT(_ref.loadRelaxed() != 0);
 #endif
 }
 
@@ -121,7 +113,6 @@ QAbstractEventDispatcher *QThreadData::createEventDispatcher()
 {
     QAbstractEventDispatcher *ed = QThreadPrivate::createEventDispatcher(this);
     eventDispatcher.storeRelease(ed);
-    ed->startingUp();
     return ed;
 }
 
@@ -138,8 +129,9 @@ QAdoptedThread::QAdoptedThread(QThreadData *data)
     d_func()->running = true;
     d_func()->finished = false;
     init();
+    d_func()->m_statusOrPendingObjects.setStatusAndClearList(
+                QtPrivate::getBindingStatus({}));
 #endif
-
     // fprintf(stderr, "new QAdoptedThread = %p\n", this);
 }
 
@@ -171,13 +163,16 @@ QThreadPrivate::QThreadPrivate(QThreadData *d)
 // to 128K.
 #ifdef Q_OS_INTEGRITY
     stackSize = 128 * 1024;
+#elif defined(Q_OS_RTEMS)
+    Q_CONSTINIT static bool envStackSizeOk = false;
+    static const int envStackSize = qEnvironmentVariableIntValue("QT_DEFAULT_THREAD_STACK_SIZE", &envStackSizeOk);
+    if (envStackSizeOk)
+        stackSize = envStackSize;
 #endif
 
 #if defined (Q_OS_WIN)
     handle = 0;
-#  ifndef Q_OS_WINRT
     id = 0;
-#  endif
     waiters = 0;
     terminationEnabled = true;
     terminatePending = false;
@@ -189,6 +184,10 @@ QThreadPrivate::QThreadPrivate(QThreadData *d)
 
 QThreadPrivate::~QThreadPrivate()
 {
+    // access to m_statusOrPendingObjects cannot race with anything
+    // unless there is already a potential use-after-free bug, as the
+    // thread is in the process of being destroyed
+    delete m_statusOrPendingObjects.list();
     data->deref();
 }
 
@@ -241,11 +240,13 @@ QThreadPrivate::~QThreadPrivate()
     different threads. Check that it is safe to do so.
 
     \note Care must be taken when interacting with objects across different
-    threads. See \l{Synchronizing Threads} for details.
+    threads. As a general rule, functions can only be called from the thread
+    that created the QThread object itself (e.g. setPriority()), unless the
+    documentation says otherwise. See \l{Synchronizing Threads} for details.
 
     \section1 Managing Threads
 
-    QThread will notifiy you via a signal when the thread is
+    QThread will notify you via a signal when the thread is
     started() and finished(), or you can use isFinished() and
     isRunning() to query the state of the thread.
 
@@ -283,11 +284,11 @@ QThreadPrivate::~QThreadPrivate()
     If you don't call \l{QObject::setObjectName()}{setObjectName()},
     the name given to your thread will be the class name of the runtime
     type of your thread object (for example, \c "RenderThread" in the case of the
-    \l{Mandelbrot Example}, as that is the name of the QThread subclass).
+    \l{Mandelbrot} example, as that is the name of the QThread subclass).
     Note that this is currently not available with release builds on Windows.
 
     \sa {Thread Support in Qt}, QThreadStorage, {Synchronizing Threads},
-        {Mandelbrot Example}, {Semaphores Example}, {Wait Conditions Example}
+        Mandelbrot, {Semaphores Example}, {Wait Conditions Example}
 */
 
 /*!
@@ -306,9 +307,20 @@ QThreadPrivate::~QThreadPrivate()
 /*!
     \fn int QThread::idealThreadCount()
 
-    Returns the ideal number of threads that can be run on the system. This is done querying
-    the number of processor cores, both real and logical, in the system. This function returns 1
-    if the number of processor cores could not be detected.
+    Returns the ideal number of threads that this process can run in parallel.
+    This is done by querying the number of logical processors available to this
+    process (if supported by this OS) or the total number of logical processors
+    in the system. This function returns 1 if neither value could be
+    determined.
+
+    \note On operating systems that support setting a thread's affinity to a
+    subset of all logical processors, the value returned by this function may
+    change between threads and over time.
+
+    \note On operating systems that support CPU hotplugging and hot-unplugging,
+    the value returned by this function may also change over time (and note
+    that CPUs can be turned on and off by software, without a physical,
+    hardware change).
 */
 
 /*!
@@ -392,8 +404,8 @@ QThreadPrivate::~QThreadPrivate()
 QThread *QThread::currentThread()
 {
     QThreadData *data = QThreadData::current();
-    Q_ASSERT(data != 0);
-    return data->thread;
+    Q_ASSERT(data != nullptr);
+    return data->thread.loadAcquire();
 }
 
 /*!
@@ -408,7 +420,7 @@ QThread::QThread(QObject *parent)
 {
     Q_D(QThread);
     // fprintf(stderr, "QThreadData %p created for thread %p\n", d->data, this);
-    d->data->thread = this;
+    d->data->thread.storeRelaxed(this);
 }
 
 /*!
@@ -419,7 +431,7 @@ QThread::QThread(QThreadPrivate &dd, QObject *parent)
 {
     Q_D(QThread);
     // fprintf(stderr, "QThreadData %p taken from private data for thread %p\n", d->data, this);
-    d->data->thread = this;
+    d->data->thread.storeRelaxed(this);
 }
 
 /*!
@@ -430,6 +442,15 @@ QThread::QThread(QThreadPrivate &dd, QObject *parent)
     isFinished() returns \c false) will result in a program
     crash. Wait for the finished() signal before deleting the
     QThread.
+
+    Since Qt 6.3, it is allowed to delete a QThread instance created by
+    a call to QThread::create() even if the corresponding thread is
+    still running. In such a case, Qt will post an interruption request
+    to that thread (via requestInterruption()); will ask the thread's
+    event loop (if any) to quit (via quit()); and will block until the
+    thread has finished.
+
+    \sa create(), isInterruptionRequested(), exec(), quit()
 */
 QThread::~QThread()
 {
@@ -444,11 +465,12 @@ QThread::~QThread()
         if (d->running && !d->finished && !d->data->isAdopted)
             qFatal("QThread: Destroyed while thread is still running");
 
-        d->data->thread = 0;
+        d->data->thread.storeRelease(nullptr);
     }
 }
 
 /*!
+    \threadsafe
     Returns \c true if the thread is finished; otherwise returns \c false.
 
     \sa isRunning()
@@ -461,6 +483,7 @@ bool QThread::isFinished() const
 }
 
 /*!
+    \threadsafe
     Returns \c true if the thread is running; otherwise returns \c false.
 
     \sa isFinished()
@@ -506,6 +529,24 @@ uint QThread::stackSize() const
     return d->stackSize;
 }
 
+/*!
+    \internal
+    Transitions BindingStatusOrList to the binding status state. If we had a list of
+    pending objects, all objects get their reinitBindingStorageAfterThreadMove method
+    called, and afterwards, the list gets discarded.
+ */
+void QtPrivate::BindingStatusOrList::setStatusAndClearList(QBindingStatus *status) noexcept
+{
+
+    if (auto pendingObjects = list()) {
+        for (auto obj: *pendingObjects)
+            QObjectPrivate::get(obj)->reinitBindingStorageAfterThreadMove();
+        delete pendingObjects;
+    }
+    // synchronizes-with the load-acquire in bindingStatus():
+    data.store(encodeBindingStatus(status), std::memory_order_release);
+}
+
 /*!
     Enters the event loop and waits until exit() is called, returning the value
     that was passed to exit(). The value returned is 0 if exit() is called via
@@ -514,12 +555,18 @@ uint QThread::stackSize() const
     This function is meant to be called from within run(). It is necessary to
     call this function to start event handling.
 
+    \note This can only be called within the thread itself, i.e. when
+    it is the current thread.
+
     \sa quit(), exit()
 */
 int QThread::exec()
 {
     Q_D(QThread);
+    const auto status = QtPrivate::getBindingStatus(QtPrivate::QBindingStatusAccessToken{});
+
     QMutexLocker locker(&d->mutex);
+    d->m_statusOrPendingObjects.setStatusAndClearList(status);
     d->data->quitNow = false;
     if (d->exited) {
         d->exited = false;
@@ -536,7 +583,60 @@ int QThread::exec()
     return returnCode;
 }
 
+
 /*!
+    \internal
+    If BindingStatusOrList is already in the binding status state, this will
+    return that BindingStatus pointer.
+    Otherwise, \a object is added to the list, and we return nullptr.
+    The list is allocated if it does not already exist.
+ */
+QBindingStatus *QtPrivate::BindingStatusOrList::addObjectUnlessAlreadyStatus(QObject *object)
+{
+    if (auto status = bindingStatus())
+        return status;
+    List *objectList = list();
+    if (!objectList) {
+        objectList = new List();
+        objectList->reserve(8);
+        data.store(encodeList(objectList), std::memory_order_relaxed);
+    }
+    objectList->push_back(object);
+    return nullptr;
+}
+
+/*!
+    \internal
+    If BindingStatusOrList is a list, remove \a object from it
+ */
+void QtPrivate::BindingStatusOrList::removeObject(QObject *object)
+{
+    List *objectList = list();
+    if (!objectList)
+        return;
+    auto it = std::remove(objectList->begin(), objectList->end(), object);
+    objectList->erase(it, objectList->end());
+}
+
+QBindingStatus *QThreadPrivate::addObjectWithPendingBindingStatusChange(QObject *obj)
+{
+    if (auto status = m_statusOrPendingObjects.bindingStatus())
+        return status;
+    QMutexLocker lock(&mutex);
+    return m_statusOrPendingObjects.addObjectUnlessAlreadyStatus(obj);
+}
+
+void QThreadPrivate::removeObjectWithPendingBindingStatusChange(QObject *obj)
+{
+    if (m_statusOrPendingObjects.bindingStatus())
+        return;
+    QMutexLocker lock(&mutex);
+    m_statusOrPendingObjects.removeObject(obj);
+}
+
+
+/*!
+    \threadsafe
     Tells the thread's event loop to exit with a return code.
 
     After calling this function, the thread leaves the event loop and
@@ -571,6 +671,7 @@ void QThread::exit(int returnCode)
 }
 
 /*!
+    \threadsafe
     Tells the thread's event loop to exit with return code 0 (success).
     Equivalent to calling QThread::exit(0).
 
@@ -607,7 +708,7 @@ void QThread::run()
     priority.
 
     The \a priority argument can be any value in the \c
-    QThread::Priority enum except for \c InheritPriorty.
+    QThread::Priority enum except for \c InheritPriority.
 
     The effect of the \a priority parameter is dependent on the
     operating system's scheduling policy. In particular, the \a priority
@@ -619,6 +720,10 @@ void QThread::run()
 */
 void QThread::setPriority(Priority priority)
 {
+    if (priority == QThread::InheritPriority) {
+        qWarning("QThread::setPriority: Argument cannot be InheritPriority");
+        return;
+    }
     Q_D(QThread);
     QMutexLocker locker(&d->mutex);
     if (!d->running) {
@@ -695,6 +800,7 @@ QThread::Priority QThread::priority() const
 
 /*!
     \fn void QThread::terminate()
+    \threadsafe
 
     Terminates the execution of the thread. The thread may or may not
     be terminated immediately, depending on the operating system's
@@ -721,7 +827,8 @@ QThread::Priority QThread::priority() const
 */
 
 /*!
-    \fn bool QThread::wait(unsigned long time)
+    \fn bool QThread::wait(QDeadlineTimer deadline)
+    \since 5.15
 
     Blocks the thread until either of these conditions is met:
 
@@ -730,12 +837,14 @@ QThread::Priority QThread::priority() const
        execution (i.e. when it returns from \l{run()}). This function
        will return true if the thread has finished. It also returns
        true if the thread has not been started yet.
-    \li \a time milliseconds has elapsed. If \a time is ULONG_MAX (the
-        default), then the wait will never timeout (the thread must
-        return from \l{run()}). This function will return false if the
-        wait timed out.
+    \li The \a deadline is reached. This function will return false if the
+       deadline is reached.
     \endlist
 
+    A deadline timer set to \c QDeadlineTimer::Forever (the default) will never
+    time out: in this case, the function only returns when the thread returns
+    from \l{run()} or if the thread has not yet started.
+
     This provides similar functionality to the POSIX \c
     pthread_join() function.
 
@@ -783,7 +892,7 @@ QThread::QThread(QObject *parent)
     : QObject(*(new QThreadPrivate), parent)
 {
     Q_D(QThread);
-    d->data->thread = this;
+    d->data->thread.storeRelaxed(this);
 }
 
 QThread::~QThread()
@@ -828,28 +937,28 @@ void QThread::exit(int returnCode)
     }
 }
 
-bool QThread::wait(unsigned long time)
+bool QThread::wait(QDeadlineTimer deadline)
 {
-    Q_UNUSED(time);
+    Q_UNUSED(deadline);
     return false;
 }
 
-bool QThread::event(QEvent* event)
+bool QThread::event(QEvent *event)
 {
     return QObject::event(event);
 }
 
-Qt::HANDLE QThread::currentThreadId() Q_DECL_NOTHROW
+Qt::HANDLE QThread::currentThreadIdImpl() noexcept
 {
     return Qt::HANDLE(currentThread());
 }
 
 QThread *QThread::currentThread()
 {
-    return QThreadData::current()->thread;
+    return QThreadData::current()->thread.loadAcquire();
 }
 
-int QThread::idealThreadCount() Q_DECL_NOTHROW
+int QThread::idealThreadCount() noexcept
 {
     return 1;
 }
@@ -870,19 +979,29 @@ bool QThread::isRunning() const
     return d->running;
 }
 
+void QThread::requestInterruption()
+{
+
+}
+
+bool QThread::isInterruptionRequested() const
+{
+    return false;
+}
+
 // No threads: so we can just use static variables
-static QThreadData *data = 0;
+Q_CONSTINIT static QThreadData *data = nullptr;
 
 QThreadData *QThreadData::current(bool createIfNecessary)
 {
     if (!data && createIfNecessary) {
         data = new QThreadData;
         data->thread = new QAdoptedThread(data);
-        data->threadId.store(Qt::HANDLE(data->thread));
+        data->threadId.storeRelaxed(Qt::HANDLE(data->thread.loadAcquire()));
         data->deref();
         data->isAdopted = true;
-        if (!QCoreApplicationPrivate::theMainThread)
-            QCoreApplicationPrivate::theMainThread = data->thread.load();
+        if (!QCoreApplicationPrivate::theMainThread.loadAcquire())
+            QCoreApplicationPrivate::theMainThread.storeRelease(data->thread.loadRelaxed());
     }
     return data;
 }
@@ -901,7 +1020,7 @@ QThread::QThread(QThreadPrivate &dd, QObject *parent)
 {
     Q_D(QThread);
     // fprintf(stderr, "QThreadData %p taken from private data for thread %p\n", d->data, this);
-    d->data->thread = this;
+    d->data->thread.storeRelaxed(this);
 }
 
 QThreadPrivate::QThreadPrivate(QThreadData *d) : data(d ? d : new QThreadData)
@@ -910,22 +1029,32 @@ QThreadPrivate::QThreadPrivate(QThreadData *d) : data(d ? d : new QThreadData)
 
 QThreadPrivate::~QThreadPrivate()
 {
-    data->thread = nullptr; // prevent QThreadData from deleting the QThreadPrivate (again).
+    data->thread.storeRelease(nullptr); // prevent QThreadData from deleting the QThreadPrivate (again).
     delete data;
 }
 
+void QThread::setStackSize(uint stackSize)
+{
+    Q_UNUSED(stackSize);
+}
+
+uint QThread::stackSize() const
+{
+    return 0;
+}
+
 #endif // QT_CONFIG(thread)
 
 /*!
     \since 5.0
 
     Returns a pointer to the event dispatcher object for the thread. If no event
-    dispatcher exists for the thread, this function returns 0.
+    dispatcher exists for the thread, this function returns \nullptr.
 */
 QAbstractEventDispatcher *QThread::eventDispatcher() const
 {
     Q_D(const QThread);
-    return d->data->eventDispatcher.load();
+    return d->data->eventDispatcher.loadRelaxed();
 }
 
 /*!
@@ -933,8 +1062,11 @@ QAbstractEventDispatcher *QThread::eventDispatcher() const
 
     Sets the event dispatcher for the thread to \a eventDispatcher. This is
     only possible as long as there is no event dispatcher installed for the
-    thread yet. That is, before the thread has been started with start() or, in
-    case of the main thread, before QCoreApplication has been instantiated.
+    thread yet.
+
+    An event dispatcher is automatically created for the main thread when \l
+    QCoreApplication is instantiated and on start() for auxiliary threads.
+
     This method takes ownership of the object.
 */
 void QThread::setEventDispatcher(QAbstractEventDispatcher *eventDispatcher)
@@ -951,6 +1083,14 @@ void QThread::setEventDispatcher(QAbstractEventDispatcher *eventDispatcher)
     }
 }
 
+/*!
+    \fn bool QThread::wait(unsigned long time)
+
+    \overload
+    \a time is the time to wait in milliseconds.
+    If \a time is ULONG_MAX, then the wait will never timeout.
+*/
+
 #if QT_CONFIG(thread)
 
 /*!
@@ -968,6 +1108,7 @@ bool QThread::event(QEvent *event)
 
 /*!
     \since 5.2
+    \threadsafe
 
     Request the interruption of the thread.
     That request is advisory and it is up to code running on the thread to decide
@@ -980,13 +1121,11 @@ bool QThread::event(QEvent *event)
 
 void QThread::requestInterruption()
 {
-    if (this == QCoreApplicationPrivate::theMainThread) {
+    if (this == QCoreApplicationPrivate::theMainThread.loadAcquire()) {
         qWarning("QThread::requestInterruption has no effect on the main thread");
         return;
     }
     Q_D(QThread);
-    // ### Qt 6: use std::atomic_flag, and document that
-    // requestInterruption/isInterruptionRequested do not synchronize with each other
     QMutexLocker locker(&d->mutex);
     if (!d->running || d->finished || d->isInFinish)
         return;
@@ -1014,6 +1153,9 @@ void QThread::requestInterruption()
     }
     \endcode
 
+    \note This can only be called within the thread itself, i.e. when
+    it is the current thread.
+
     \sa currentThread() requestInterruption()
 */
 bool QThread::isInterruptionRequested() const
@@ -1043,29 +1185,6 @@ bool QThread::isInterruptionRequested() const
 
     \note the caller acquires ownership of the returned QThread instance.
 
-    \note this function is only available when using C++17.
-
-    \warning do not call start() on the returned QThread instance more than once;
-    doing so will result in undefined behavior.
-
-    \sa start()
-*/
-
-/*!
-    \fn template <typename Function> QThread *QThread::create(Function &&f)
-    \since 5.10
-
-    Creates a new QThread object that will execute the function \a f.
-
-    The new thread is not started -- it must be started by an explicit call
-    to start(). This allows you to connect to its signals, move QObjects
-    to the thread, choose the new thread's priority and so on. The function
-    \a f will be called in the new thread.
-
-    Returns the newly created QThread instance.
-
-    \note the caller acquires ownership of the returned QThread instance.
-
     \warning do not call start() on the returned QThread instance more than once;
     doing so will result in undefined behavior.
 
@@ -1081,6 +1200,13 @@ public:
     {
     }
 
+    ~QThreadCreateThread()
+    {
+        requestInterruption();
+        quit();
+        wait();
+    }
+
 private:
     void run() override
     {
