@@ -1,31 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2022 The Qt Company Ltd.
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <QTest>
 
@@ -37,6 +12,8 @@
 
 #include "../shared/test_number_shared.h"
 
+using namespace Qt::StringLiterals;
+
 class tst_QByteArray : public QObject
 {
     Q_OBJECT
@@ -48,13 +25,6 @@ private slots:
     void swap();
     void qChecksum_data();
     void qChecksum();
-    void qCompress_data();
-#ifndef QT_NO_COMPRESS
-    void qCompress();
-    void qUncompressCorruptedData_data();
-    void qUncompressCorruptedData();
-    void qCompressionZeroTermination();
-#endif
     void constByteArray();
     void leftJustified();
     void rightJustified();
@@ -78,6 +48,7 @@ private slots:
     void prependExtended_data();
     void prependExtended();
     void append();
+    void appendFromRawData();
     void appendExtended_data();
     void appendExtended();
     void insert();
@@ -85,7 +56,10 @@ private slots:
     void insertExtended();
     void remove_data();
     void remove();
+    void remove_extra();
     void removeIf();
+    void erase();
+    void erase_single_arg();
     void replace_data();
     void replace();
     void replaceWithSpecifiedLength();
@@ -125,8 +99,9 @@ private slots:
     void reserve();
     void reserveExtended_data();
     void reserveExtended();
-    void movablity_data();
-    void movablity();
+    void resize();
+    void movability_data();
+    void movability();
     void literals();
     void userDefinedLiterals();
     void toUpperLower_data();
@@ -174,7 +149,7 @@ QByteArray verifyZeroTermination(const QByteArray &ba)
     if (!baDataPtr->isMutable())
         return ba;
 
-    int baSize = ba.size();
+    qsizetype baSize = ba.size();
     char baTerminator = ba.constData()[baSize];
     if ('\0' != baTerminator)
         return QString::fromUtf8(
@@ -248,86 +223,13 @@ void tst_QByteArray::qChecksum()
     QFETCH(Qt::ChecksumType, standard);
     QFETCH(uint, checksum);
 
-    QCOMPARE(data.length(), int(len));
+    QCOMPARE(data.size(), int(len));
     if (standard == Qt::ChecksumIso3309) {
         QCOMPARE(::qChecksum(QByteArrayView(data.constData(), len)), static_cast<quint16>(checksum));
     }
     QCOMPARE(::qChecksum(QByteArrayView(data.constData(), len), standard), static_cast<quint16>(checksum));
 }
 
-void tst_QByteArray::qCompress_data()
-{
-    QTest::addColumn<QByteArray>("ba");
-
-    const int size1 = 1024*1024;
-    QByteArray ba1( size1, 0 );
-
-    QTest::newRow( "00" ) << QByteArray();
-
-    int i;
-    for ( i=0; i<size1; i++ )
-        ba1[i] = (char)( i / 1024 );
-    QTest::newRow( "01" ) << ba1;
-
-    for ( i=0; i<size1; i++ )
-        ba1[i] = (char)( i % 256 );
-    QTest::newRow( "02" ) << ba1;
-
-    ba1.fill( 'A' );
-    QTest::newRow( "03" ) << ba1;
-
-    QFile file( QFINDTESTDATA("rfc3252.txt") );
-    QVERIFY( file.open(QIODevice::ReadOnly) );
-    QTest::newRow( "04" ) << file.readAll();
-}
-
-#ifndef QT_NO_COMPRESS
-void tst_QByteArray::qCompress()
-{
-    QFETCH( QByteArray, ba );
-    QByteArray compressed = ::qCompress( ba );
-    QTEST( ::qUncompress( compressed ), "ba" );
-}
-
-void tst_QByteArray::qUncompressCorruptedData_data()
-{
-    QTest::addColumn<QByteArray>("in");
-
-    QTest::newRow("0x00000000") << QByteArray("\x00\x00\x00\x00", 4);
-    QTest::newRow("0x000000ff") << QByteArray("\x00\x00\x00\xff", 4);
-    QTest::newRow("0x3f000000") << QByteArray("\x3f\x00\x00\x00", 4);
-    QTest::newRow("0x3fffffff") << QByteArray("\x3f\xff\xff\xff", 4);
-    QTest::newRow("0x7fffff00") << QByteArray("\x7f\xff\xff\x00", 4);
-    QTest::newRow("0x7fffffff") << QByteArray("\x7f\xff\xff\xff", 4);
-    QTest::newRow("0x80000000") << QByteArray("\x80\x00\x00\x00", 4);
-    QTest::newRow("0x800000ff") << QByteArray("\x80\x00\x00\xff", 4);
-    QTest::newRow("0xcf000000") << QByteArray("\xcf\x00\x00\x00", 4);
-    QTest::newRow("0xcfffffff") << QByteArray("\xcf\xff\xff\xff", 4);
-    QTest::newRow("0xffffff00") << QByteArray("\xff\xff\xff\x00", 4);
-    QTest::newRow("0xffffffff") << QByteArray("\xff\xff\xff\xff", 4);
-}
-
-// This test is expected to produce some warning messages in the test output.
-void tst_QByteArray::qUncompressCorruptedData()
-{
-    QFETCH(QByteArray, in);
-
-    QByteArray res;
-    res = ::qUncompress(in);
-    QCOMPARE(res, QByteArray());
-
-    res = ::qUncompress(in + "blah");
-    QCOMPARE(res, QByteArray());
-}
-
-void tst_QByteArray::qCompressionZeroTermination()
-{
-    QByteArray s = "Hello, I'm a string.";
-    QByteArray ba = ::qUncompress(::qCompress(s));
-    QCOMPARE(ba.data()[ba.size()], '\0');
-    QCOMPARE(ba, s);
-}
-#endif
 
 void tst_QByteArray::constByteArray()
 {
@@ -518,7 +420,7 @@ void tst_QByteArray::split()
     QFETCH(int, size);
 
     QList<QByteArray> list = sample.split(' ');
-    QCOMPARE(list.count(), size);
+    QCOMPARE(list.size(), size);
 }
 
 void tst_QByteArray::swap()
@@ -841,7 +743,10 @@ void tst_QByteArray::qstrncpy()
 
     // src == nullptr
     QCOMPARE(::qstrncpy(dst.data(), 0,  0), (char*)0);
+    QCOMPARE(*dst.data(), 'b'); // must not have written to dst
     QCOMPARE(::qstrncpy(dst.data(), 0, 10), (char*)0);
+    QCOMPARE(*dst.data(), '\0'); // must have written to dst
+    *dst.data() = 'b'; // restore
 
     // valid pointers, but len == 0
     QCOMPARE(::qstrncpy(dst.data(), src.data(), 0), dst.data());
@@ -1002,6 +907,20 @@ void tst_QByteArray::append()
     }
 }
 
+void tst_QByteArray::appendFromRawData()
+{
+    char rawData[] = "Hello World!";
+    QByteArray ba = QByteArray::fromRawData(rawData, std::size(rawData) - 1);
+
+    QByteArray copy;
+    copy.append(ba);
+    QCOMPARE(copy, ba);
+    // We make an _actual_ copy, because appending a byte array
+    // created with fromRawData() might be optimized to copy the DataPointer,
+    // which means we may point to temporary stack data.
+    QCOMPARE_NE((void *)copy.constData(), (void *)ba.constData());
+}
+
 void tst_QByteArray::appendExtended_data()
 {
     prependExtended_data();
@@ -1162,7 +1081,30 @@ void tst_QByteArray::remove()
     QFETCH(int, position);
     QFETCH(int, length);
     QFETCH(QByteArray, expected);
-    QCOMPARE(src.remove(position, length), expected);
+    // Test when it's shared
+    QByteArray ba1 = src;
+    QCOMPARE(ba1.remove(position, length), expected);
+
+    // Test when it's not shared
+    QByteArray ba2 = src;
+    ba2.detach();
+    QCOMPARE(ba2.remove(position, length), expected);
+}
+
+void tst_QByteArray::remove_extra()
+{
+    QByteArray ba = "Clock";
+    ba.removeFirst();
+    QCOMPARE(ba, "lock");
+    ba.removeLast();
+    QCOMPARE(ba, "loc");
+    ba.removeAt(ba.indexOf('o'));
+    QCOMPARE(ba, "lc");
+    ba.clear();
+    // No crash on empty byte arrays
+    ba.removeFirst();
+    ba.removeLast();
+    ba.removeAt(2);
 }
 
 void tst_QByteArray::removeIf()
@@ -1177,6 +1119,49 @@ void tst_QByteArray::removeIf()
     QCOMPARE(a.removeIf(removeA), QByteArray("BcbC"));
 }
 
+void tst_QByteArray::erase()
+{
+    {
+        QByteArray ba = "kittens";
+        auto it = ba.erase(ba.cbegin(), ba.cbegin() + 2);
+        QCOMPARE(ba, "ttens");
+        QCOMPARE(it, ba.cbegin());
+    }
+
+    {
+        QByteArray ba = "kittens";
+        auto it = ba.erase(ba.cbegin(), ba.cend());
+        QCOMPARE(ba, "");
+        QCOMPARE(it, ba.cbegin());
+        QCOMPARE(ba.cbegin(), ba.cend());
+    }
+
+    {
+        QByteArray ba = "kite";
+        auto it = ba.erase(ba.cbegin(), ba.cbegin());
+        // erase() should return an iterator (not const_iterator)
+        *it = 'Z';
+        QCOMPARE(ba, "Zite");
+        QCOMPARE(it, ba.cbegin());
+    }
+}
+
+void tst_QByteArray::erase_single_arg()
+{
+    QByteArray ba = "abcdefg";
+    ba.erase(ba.cend());
+    auto it = ba.erase(ba.cbegin());
+    QCOMPARE_EQ(ba, "bcdefg");
+    QCOMPARE(it, ba.cbegin());
+
+    it = ba.erase(std::prev(ba.end()));
+    QCOMPARE_EQ(ba, "bcdef");
+    QCOMPARE(it, ba.cend());
+
+    it = ba.erase(std::find(ba.begin(), ba.end(), QChar('d')));
+    QCOMPARE(it, ba.begin() + 2);
+}
+
 void tst_QByteArray::replace_data()
 {
     // Try to cover both the index and specific char cases.
@@ -1291,7 +1276,7 @@ void tst_QByteArray::number_double_data()
         QTest::addRow("%s, format '%c', precision %d", title, datum.f, datum.p)
                 << datum.d << datum.f << datum.p << ba;
         if (datum.f != 'f') { // Also test uppercase format
-            datum.f = toupper(datum.f);
+            datum.f = QtMiscUtils::toAsciiUpper(datum.f);
             QByteArray upper = ba.toUpper();
             QByteArray upperTitle = QByteArray(title);
             if (!datum.optTitle.isEmpty())
@@ -1590,7 +1575,7 @@ void tst_QByteArray::toFromHex_data()
         << QByteArray("af")
         << QByteArray("xaf");
 
-    QTest::newRow("no-leading-zero")
+    QTest::newRow("no-leading-zero-long")
         << QByteArray("\xd\xde\xad\xc0\xde")
         << '\0'
         << QByteArray("0ddeadc0de")
@@ -1652,15 +1637,15 @@ void tst_QByteArray::toFromPercentEncoding()
 
     QByteArray data = arr.toPercentEncoding();
     QCOMPARE(data, QByteArray("Qt%20is%20great%21"));
-    QCOMPARE(QByteArray::fromPercentEncoding(data), arr);
+    QCOMPARE(data.percentDecoded(), arr);
 
     data = arr.toPercentEncoding("! ", "Qt");
     QCOMPARE(data, QByteArray("%51%74 is grea%74!"));
-    QCOMPARE(QByteArray::fromPercentEncoding(data), arr);
+    QCOMPARE(data.percentDecoded(), arr);
 
     data = arr.toPercentEncoding(QByteArray(), "abcdefghijklmnopqrstuvwxyz", 'Q');
     QCOMPARE(data, QByteArray("Q51Q74Q20Q69Q73Q20Q67Q72Q65Q61Q74Q21"));
-    QCOMPARE(QByteArray::fromPercentEncoding(data, 'Q'), arr);
+    QCOMPARE(data.percentDecoded('Q'), arr);
 
     // verify that to/from percent encoding preserves nullity
     arr = "";
@@ -1676,8 +1661,16 @@ void tst_QByteArray::toFromPercentEncoding()
     QVERIFY(arr.isNull());
     QVERIFY(arr.toPercentEncoding().isEmpty());
     QVERIFY(arr.toPercentEncoding().isNull());
-    QVERIFY(QByteArray::fromPercentEncoding(QByteArray()).isEmpty());
-    QVERIFY(QByteArray::fromPercentEncoding(QByteArray()).isNull());
+    QVERIFY(QByteArray().percentDecoded().isEmpty());
+    QVERIFY(QByteArray().percentDecoded().isNull());
+
+    // Verify that literal % in the string to be encoded does round-trip:
+    arr = "Qt%20is%20great%21";
+    data = arr.toPercentEncoding();
+    QCOMPARE(data.percentDecoded(), arr);
+    arr = "87% of all statistics are made up!";
+    data = arr.toPercentEncoding();
+    QCOMPARE(data.percentDecoded(), arr);
 }
 
 void tst_QByteArray::fromPercentEncoding_data()
@@ -1699,7 +1692,7 @@ void tst_QByteArray::fromPercentEncoding()
     QFETCH(QByteArray, encodedString);
     QFETCH(QByteArray, decodedString);
 
-    QCOMPARE(QByteArray::fromPercentEncoding(encodedString), decodedString);
+    QCOMPARE(encodedString.percentDecoded(), decodedString);
 }
 
 void tst_QByteArray::toPercentEncoding_data()
@@ -1760,7 +1753,7 @@ void tst_QByteArray::pecentEncodingRoundTrip()
 
     QByteArray encodedData = original.toPercentEncoding(excludeInEncoding, includeInEncoding);
     QCOMPARE(encodedData, encoded);
-    QCOMPARE(QByteArray::fromPercentEncoding(encodedData), original);
+    QCOMPARE(encodedData.percentDecoded(), original);
 }
 
 struct StringComparisonData
@@ -2083,9 +2076,26 @@ void tst_QByteArray::reserveExtended()
     QCOMPARE(array.capacity(), array.size());
 }
 
-void tst_QByteArray::movablity_data()
+void tst_QByteArray::resize()
 {
-    QTest::addColumn<QByteArray>("array");
+    QByteArray ba;
+    ba.resize(15);
+    QCOMPARE(ba.size(), qsizetype(15));
+    ba.resize(10);
+    QCOMPARE(ba.size(), 10);
+    ba.resize(0);
+    QCOMPARE(ba.size(), 0);
+    ba.resize(5, 'a');
+    QCOMPARE(ba.size(), 5);
+    QCOMPARE(ba, "aaaaa");
+    ba.resize(10, 'b');
+    QCOMPARE(ba.size(), 10);
+    QCOMPARE(ba, "aaaaabbbbb");
+}
+
+void tst_QByteArray::movability_data()
+{
+    prependExtended_data();
 
     QTest::newRow("0x00000000") << QByteArray("\x00\x00\x00\x00", 4);
     QTest::newRow("0x000000ff") << QByteArray("\x00\x00\x00\xff", 4);
@@ -2093,11 +2103,9 @@ void tst_QByteArray::movablity_data()
     QTest::newRow("empty") << QByteArray("");
     QTest::newRow("null") << QByteArray();
     QTest::newRow("sss") << QByteArray(3, 's');
-
-    prependExtended_data();
 }
 
-void tst_QByteArray::movablity()
+void tst_QByteArray::movability()
 {
     QFETCH(QByteArray, array);
 
@@ -2168,7 +2176,7 @@ void tst_QByteArray::literals()
 {
     QByteArray str(QByteArrayLiteral("abcd"));
 
-    QVERIFY(str.length() == 4);
+    QVERIFY(str.size() == 4);
     QCOMPARE(str.capacity(), 0);
     QVERIFY(str == "abcd");
     QVERIFY(!str.data_ptr()->isMutable());
@@ -2180,34 +2188,60 @@ void tst_QByteArray::literals()
 
     // detach on non const access
     QVERIFY(str.data() != s);
-    QVERIFY(str.capacity() >= str.length());
+    QVERIFY(str.capacity() >= str.size());
 
     QVERIFY(str2.constData() == s);
     QVERIFY(str2.data() != s);
-    QVERIFY(str2.capacity() >= str2.length());
+    QVERIFY(str2.capacity() >= str2.size());
 }
 
 void tst_QByteArray::userDefinedLiterals()
 {
-    QByteArray str = "abcd"_qba;
+    {
+        QByteArray str = "abcd"_ba;
 
-    QVERIFY(str.length() == 4);
-    QCOMPARE(str.capacity(), 0);
-    QVERIFY(str == "abcd");
-    QVERIFY(!str.data_ptr()->isMutable());
+        QVERIFY(str.size() == 4);
+        QCOMPARE(str.capacity(), 0);
+        QVERIFY(str == "abcd");
+        QVERIFY(!str.data_ptr()->isMutable());
 
-    const char *s = str.constData();
-    QByteArray str2 = str;
-    QVERIFY(str2.constData() == s);
-    QCOMPARE(str2.capacity(), 0);
+        const char *s = str.constData();
+        QByteArray str2 = str;
+        QVERIFY(str2.constData() == s);
+        QCOMPARE(str2.capacity(), 0);
 
-    // detach on non const access
-    QVERIFY(str.data() != s);
-    QVERIFY(str.capacity() >= str.length());
+        // detach on non const access
+        QVERIFY(str.data() != s);
+        QVERIFY(str.capacity() >= str.size());
 
-    QVERIFY(str2.constData() == s);
-    QVERIFY(str2.data() != s);
-    QVERIFY(str2.capacity() >= str2.length());
+        QVERIFY(str2.constData() == s);
+        QVERIFY(str2.data() != s);
+        QVERIFY(str2.capacity() >= str2.size());
+    }
+
+#if QT_DEPRECATED_SINCE(6, 8)
+    {
+        QT_IGNORE_DEPRECATIONS(QByteArray str = "abcd"_qba;)
+
+        QVERIFY(str.size() == 4);
+        QCOMPARE(str.capacity(), 0);
+        QVERIFY(str == "abcd");
+        QVERIFY(!str.data_ptr()->isMutable());
+
+        const char *s = str.constData();
+        QByteArray str2 = str;
+        QVERIFY(str2.constData() == s);
+        QCOMPARE(str2.capacity(), 0);
+
+        // detach on non const access
+        QVERIFY(str.data() != s);
+        QVERIFY(str.capacity() >= str.size());
+
+        QVERIFY(str2.constData() == s);
+        QVERIFY(str2.data() != s);
+        QVERIFY(str2.capacity() >= str2.size());
+    }
+#endif // QT_DEPRECATED_SINCE(6, 8)
 }
 
 void tst_QByteArray::toUpperLower_data()
@@ -2336,7 +2370,7 @@ void tst_QByteArray::stdString()
     std::string stdstr( "QByteArray" );
 
     const QByteArray stlqt = QByteArray::fromStdString(stdstr);
-    QCOMPARE(stlqt.length(), int(stdstr.length()));
+    QCOMPARE(stlqt.size(), int(stdstr.length()));
     QCOMPARE(stlqt.data(), stdstr.c_str());
     QCOMPARE(stlqt.toStdString(), stdstr);
 
@@ -2513,9 +2547,14 @@ void tst_QByteArray::length()
     QFETCH(QByteArray, src);
     QFETCH(qsizetype, res);
 
-    QCOMPARE(src.length(), res);
     QCOMPARE(src.size(), res);
-    QCOMPARE(src.count(), res);
+    QCOMPARE(src.size(), res);
+#if QT_DEPRECATED_SINCE(6, 4)
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_DEPRECATED
+    QCOMPARE(src.size(), res);
+QT_WARNING_POP
+#endif
 }
 
 void tst_QByteArray::length_data()
